Package: lefko3
Type: Package
Title: Historical and Ahistorical Population Projection Matrix Analysis
Version: 6.7.3
Date: 2026-04-24
Authors@R: c(person(given = "Richard P.", family = "Shefferson",
             email = "cdorm@g.ecc.u-tokyo.ac.jp", role = c("aut", "cre"),
             comment=c(ORCID="0000-0002-5234-3131")),
             person(given = "Johan", family = "Ehrlen", role = "aut",
             comment=c(ORCID="0000-0001-8539-8967")))
Description: Complete analytical environment for the construction and analysis
             of matrix population models and integral projection models.
             Includes the ability to construct historical matrices, which are
             2d matrices comprising 3 consecutive times of demographic
             information. Estimates both raw and function-based forms of
             historical and standard ahistorical matrices. It also estimates
             function-based age-by-stage matrices and raw and function-based
             Leslie matrices.
Encoding: UTF-8
License: GPL (>= 2)
URL: https://github.com/dormancy1/lefko3
Imports: Rcpp (>= 1.0.5), glmmTMB, lme4, MASS, Matrix, methods, MuMIn,
        pscl, rlang, stats, VGAM, grDevices
LinkingTo: Rcpp, RcppArmadillo, BH
LazyData: true
BugReports: https://github.com/dormancy1/lefko3/issues
RoxygenNote: 7.3.3
Suggests: knitr, popbio, rmarkdown, Rcompadre
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-04-24 04:51:14 UTC; richpshefferson
Author: Richard P. Shefferson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5234-3131>),
  Johan Ehrlen [aut] (ORCID: <https://orcid.org/0000-0001-8539-8967>)
Maintainer: Richard P. Shefferson <cdorm@g.ecc.u-tokyo.ac.jp>
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-04-24 09:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-24 13:33:49 UTC; unix
Archs: lefko3.so.dSYM
