# Generated by roxygen2: do not edit by hand

S3method(elasticity3,dgCMatrix)
S3method(elasticity3,lefkoMat)
S3method(elasticity3,lefkoMatList)
S3method(elasticity3,list)
S3method(elasticity3,matrix)
S3method(image3,dgCMatrix)
S3method(image3,lefkoElas)
S3method(image3,lefkoMat)
S3method(image3,lefkoSens)
S3method(image3,list)
S3method(image3,matrix)
S3method(plot,lefkoProj)
S3method(repvalue3,dgCMatrix)
S3method(repvalue3,lefkoMat)
S3method(repvalue3,lefkoMatList)
S3method(repvalue3,list)
S3method(repvalue3,matrix)
S3method(sensitivity3,dgCMatrix)
S3method(sensitivity3,lefkoMat)
S3method(sensitivity3,lefkoMatList)
S3method(sensitivity3,list)
S3method(sensitivity3,matrix)
S3method(stablestage3,dgCMatrix)
S3method(stablestage3,lefkoMat)
S3method(stablestage3,lefkoMatList)
S3method(stablestage3,list)
S3method(stablestage3,matrix)
S3method(summary,lefkoCondMat)
S3method(summary,lefkoElas)
S3method(summary,lefkoLTRE)
S3method(summary,lefkoMat)
S3method(summary,lefkoMatList)
S3method(summary,lefkoMod)
S3method(summary,lefkoProj)
export(actualstage3)
export(add_lM)
export(add_stage)
export(aflefko2)
export(append_lP)
export(arlefko2)
export(beverton3)
export(bootstrap3)
export(cond_diff)
export(cond_hmpm)
export(create_lM)
export(create_pm)
export(cycle_check)
export(delete_lM)
export(density_input)
export(density_vr)
export(diff_lM)
export(edit_lM)
export(elasticity3)
export(f_projection3)
export(flefko2)
export(flefko3)
export(fleslie)
export(hfv_qc)
export(hist_null)
export(historicalize3)
export(image3)
export(lambda3)
export(lmean)
export(logistic3)
export(ltre3)
export(markov_run)
export(matrix_interp)
export(miniMod)
export(modelsearch)
export(mpm_create)
export(overwrite)
export(projection3)
export(repvalue3)
export(ricker3)
export(rlefko2)
export(rlefko3)
export(rleslie)
export(sensitivity3)
export(sf_create)
export(sf_distrib)
export(sf_skeleton)
export(slambda3)
export(stablestage3)
export(stage_weight)
export(start_input)
export(subset_lM)
export(summary_hfv)
export(sup_skeleton)
export(supplemental)
export(usher3)
export(verticalize3)
export(vrm_import)
import(Rcpp)
importFrom(MASS,glm.nb)
importFrom(Matrix,colSums)
importFrom(MuMIn,dredge)
importFrom(Rcpp,evalCpp)
importFrom(VGAM,posnegbinomial)
importFrom(VGAM,pospoisson)
importFrom(VGAM,predictvglm)
importFrom(VGAM,vglm)
importFrom(glmmTMB,glmmTMB)
importFrom(glmmTMB,nbinom2)
importFrom(glmmTMB,truncated_nbinom2)
importFrom(glmmTMB,truncated_poisson)
importFrom(grDevices,palette)
importFrom(grDevices,xy.coords)
importFrom(graphics,lines)
importFrom(lme4,glmer)
importFrom(lme4,lmer)
importFrom(methods,as)
importFrom(methods,is)
importFrom(pscl,zeroinfl)
importFrom(rlang,expr)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,getCall)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,na.action)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,pchisq)
importFrom(stats,poisson)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,xtabs)
useDynLib(lefko3)
